@@ -40,10 +40,10 @@
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE. See the GNU General Public License for more details.
 *
-*    You should have received a copy of the GNU General Public License
-*    along with this program; if not, write to the Free Software
-*    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
-*    USA.
+*     You should have received a copy of the GNU General Public License
+*     along with this program; if not, write to the Free Software
+*     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
+*     USA.
 
 *  Bugs:
 *     {note_any_bugs_here}
@@ -59,7 +59,8 @@
 
 static int verbose = 1;
 
-/* Support functions to allow to test results. viv and vvd match the SOFA implementations */
+/* Support functions to allow to test results.
+   viv and vvd match the SOFA/ERFA implementations */
 
 static void viv(int ival, int ivalok, const char *func, const char *test,
                 int *status)
@@ -225,6 +226,33 @@ static void t_afin( int * status ) {
   viv ( j, 0, "palDafin", "J", status );
 }
 
+/* Altaz */
+
+static void t_altaz( int *status ) {
+  double az, azd, azdd, el, eld, eldd, pa, pad, padd;
+  palAltaz( 0.7, -0.7, -0.65,
+            &az, &azd, &azdd, &el, &eld, &eldd, &pa, &pad, &padd );
+
+  vvd ( az, 4.400560746660174, 1e-12, "palAltaz",
+        "AZ", status );
+  vvd ( azd, -0.2015438937145421, 1e-13, "palAltaz",
+        "AZD", status );
+  vvd ( azdd, -0.4381266949668748, 1e-13, "palAltaz",
+        "AZDD", status );
+  vvd ( el, 1.026646506651396, 1e-12, "palAltaz",
+        "EL", status );
+  vvd ( eld, -0.7576920683826450, 1e-13, "palAltaz",
+        "ELD", status );
+  vvd ( eldd, 0.04922465406857453, 1e-14, "palAltaz",
+        "ELDD", status );
+  vvd ( pa, 1.707639969653937, 1e-12, "palAltaz",
+        "PA", status );
+  vvd ( pad, 0.4717832355365627, 1e-13, "palAltaz",
+        "PAD", status );
+  vvd ( padd, -0.2957914128185515, 1e-13, "palAltaz",
+        "PADD", status );
+}
+
 /* Airmass */
 
 static void t_airmas( int *status ) {
@@ -247,8 +275,8 @@ static void t_amp ( int *status ) {
 
   /* This is the palMapqk test */
   palAmp( 1.234, -0.567, 55927.0, 2010.0, &rm, &dm );
-  vvd( rm, 1.2335120411026936349, 1.0E-12, "palAmp", "rm", status );
-  vvd( dm, -0.56702908706930343907, 1.0E-12, "palAmp", "dm", status );
+  vvd( rm, 1.233512033578303857, 1.0E-12, "palAmp", "rm", status );
+  vvd( dm, -0.56702909748530827549, 1.0E-12, "palAmp", "dm", status );
 }
 
 /* Apparent to Observed place */
@@ -666,7 +694,7 @@ static void t_epj2d( int *status ) {
 
 /* Equation of the equinoxes */
 
-/* Use SOFA test because of change in precession model */
+/* Use SOFA/ERFA test because of change in precession model */
 static void t_eqeqx (int *status ) {
   vvd ( palEqeqx( 53736. ), -0.8834195072043790156e-5,
         1e-15, "palEqeqx", " ", status );
@@ -738,7 +766,7 @@ static void t_geoc( int *status ) {
   palGeoc( lat, alt, &r, &z );
 
   /* Note the lower tolerance than normal since the models in SLA
-     differ from the more up to date model in SOFA */
+     differ from the more up to date model in SOFA/ERFA */
   vvd( r, 4.01502667039618e-05, 1e-10, "palGeoc", "R", status );
   vvd( z, 1.43762411970295e-05, 1e-10, "palGeoc", "Z", status );
 
@@ -759,7 +787,7 @@ static void t_ge50 ( int *status ) {
 
 /* GMST */
 
-/* We use the SOFA test values rather than the values from SLA
+/* We use the SOFA/ERFA test values rather than the values from SLA
    because the precession models have changed */
 
 static void t_gmst( int *status ) {
@@ -862,7 +890,7 @@ static void t_nut( int *status ) {
   palNut( 46012.32, rmatn );
   vrmat( rmatn, expected, "palNut", 1.0e-3, status );
 
-  /* Use the SOFA tests */
+  /* Use the SOFA/ERFA tests */
   palNutc( 54388.0, &dpsi, &deps, &eps0 );
   vvd( eps0, 0.4090749229387258204, 1e-14,
       "palNutc", "eps0", status);
@@ -1081,6 +1109,16 @@ static void t_vecmat( int * status ) {
 
 }
 
+static void t_ecleq( int *status ) {
+  double dr;
+  double dd;
+  palEcleq( 1.234, -0.123, 43210.0, &dr, &dd );
+  vvd( dr, 1.229910118208851, 1e-5, "palEcleq",
+       "RA", status );
+  vvd( dd, 0.2638461400411088, 1e-5, "palEcleq",
+       "Dec", status );
+}
+
 static void t_ecmat( int *status ) {
    double rmat[3][3];
    double expected[3][3] = {
@@ -1195,7 +1233,7 @@ static void t_map( int *status ) {
            0.123, 32.1, 1999, 43210.9, &ra, &da );
 
   /* These are the SLA tests but and they agree to 0.1 arcsec
-     with PAL/SOFA. We expect a slight difference from the change
+     with PAL/SOFA/ERFA. We expect a slight difference from the change
      to nutation models. */
   vvd ( ra, 6.117130429775647, 1e-6, "palMap",
           "RA", status );
@@ -1231,26 +1269,77 @@ static void t_mappa( int *status ) {
    vvec( 21, amprms, expected, "palMappa", status );
 }
 
+static void t_mapqk( int *status ) {
+    /* Test mapqk by taking the geocentric apparent positions of Arcturus
+       as downloaded from aa.usno.mil/data/docs/geocentric.php and trying
+       to calculate it from Arcturus' mean position, proper motion, parallax,
+       and radial velocity */
+
+    double amprms[21];
+    double ra_0, dec_0; /* mean position */
+    double ra_app, dec_app; /* geocentric apparent position */
+    double ra_test, dec_test;
+    double px, pm_ra, pm_dec, v_rad;
+    int j;
+    int iposn;
+
+    /*
+    The data below represents the position of Arcturus on
+    JD (UT) 2457000.375 as reported by
+    http://aa.usno.navy.mil/data/docs/geocentric.php
+    */
+
+    const char radec_0[] = "14 15 39.67207 19 10 56.673";
+    const char radec_app[] = "14 16 19.59 19 6 19.56";
+
+    iposn = 1;
+    palDafin(radec_0, &iposn, &ra_0, &j);
+    palDafin(radec_0, &iposn, &dec_0, &j);
+    ra_0 *= 15.0;
+
+    pm_ra = -1.0939*PAL__DAS2R;
+    pm_ra /= cos(dec_0);
+    pm_dec = -2.00006*PAL__DAS2R;
+    v_rad = -5.19;
+    px = 0.08883*PAL__DAS2R;
+
+    palMappa(2000.0, 56999.87537249177, amprms);  /* time is the TDB MJD calculated from
+                                                     a JD of 2457000.375 with astropy.time */
+
+    palMapqk(ra_0, dec_0, pm_ra, pm_dec, px, v_rad, amprms, &ra_test, &dec_test);
+
+    iposn = 1;
+    palDafin(radec_app, &iposn, &ra_app, &j);
+    palDafin(radec_app, &iposn, &dec_app, &j);
+    ra_app *= 15.0;
+
+    /* find the angular distance from the known mean position
+       to the calculated mean postion */
+    double dd;
+    dd = palDsep(ra_test, dec_test, ra_app, dec_app);
+    dd *= PAL__DR2AS;
+    vvd( dd, 0.0, 0.1, "palMapqk", "distance", status);
+}
+
 static void t_mapqkz( int *status ) {
-   double amprms[21],  ra, da;
+   double amprms[21],  ra, da, ra_c, da_c;
+
+   /* Run inputs through mapqk with zero proper motion, parallax
+      and radial velocity.  Then run the same inputs through mapqkz.
+      Verify that the results are the same */
    palMappa( 2010.0, 55927.0, amprms );
+   palMapqk(1.234, -0.567, 0.0, 0.0, 0.0, 0.0, amprms, &ra_c, &da_c);
    palMapqkz( 1.234, -0.567, amprms, &ra, &da );
-   vvd( ra, 1.2344879748414849807, 1.0E-12, "palMapqkz", "ra", status );
-   vvd( da, -0.56697099554368701746, 1.0E-12, "palMapqkz", "da", status );
-
-   /* Try the same with palMapqk and zero parallax and proper motion */
-   palMapqk( 1.234, -0.567, 0., 0., 0., 0., amprms, &ra, &da );
-   vvd( ra, 1.2344879748414849807, 1.0E-7, "palMapqkz", "ra", status );
-   vvd( da, -0.56697099554368701746, 1.0E-7, "palMapqkz", "da", status );
-
+   vvd( ra, ra_c, 1.0E-12, "palMapqkz", "ra", status );
+   vvd( da, da_c, 1.0E-12, "palMapqkz", "da", status );
 }
 
 static void t_ampqk( int *status ) {
    double amprms[21],  rm, dm;
    palMappa( 2010.0, 55927.0, amprms );
    palAmpqk( 1.234, -0.567, amprms, &rm, &dm );
-   vvd( rm, 1.2335120411026936349, 1.0E-12, "palAmpqk", "rm", status );
-   vvd( dm, -0.56702908706930343907, 1.0E-12, "palAmpqk", "dm", status );
+   vvd( rm, 1.233512033578303857, 1.0E-12, "palAmpqk", "rm", status );
+   vvd( dm, -0.56702909748530827549, 1.0E-12, "palAmpqk", "dm", status );
 }
 
 static void t_fk45z( int *status ) {
@@ -1269,6 +1358,18 @@ static void t_fk54z( int *status ) {
    vvd( dd1950, 7.196059425334821089e-09, 1.0E-12, "palFk54z", "dd1950", status );
 }
 
+static void t_fk524( int *status ) {
+  double r1950, d1950, dr1950, dd1950, p1950, v1950;
+  palFk524(4.567, -1.23, -3e-5, 8e-6, 0.29,
+           -35.0, &r1950, &d1950, &dr1950, &dd1950, &p1950, &v1950);
+  vvd(r1950, 4.543778603272084, 1e-12, "palFk524", "r", status);
+  vvd(d1950, -1.229642790187574, 1e-12, "palFk524", "d", status);
+  vvd(dr1950, -2.957873121769244e-5, 1e-17, "palFk524", "dr", status);
+  vvd(dd1950, 8.117725309659079e-6, 1e-17, "palFk524", "dd", status);
+  vvd(p1950, 0.2898494999992917, 1e-12, "palFk524", "p", status);
+  vvd(v1950, -35.026862824252680, 1e-11, "palFk524", "v", status);
+}
+
 static void t_flotin( int * status ) {
 
   int j;
@@ -1402,6 +1503,30 @@ static void t_pa( int *status ) {
         0, 0, "palPa", "zenith", status );
 }
 
+static void t_pcd( int *status ) {
+  double disco, x, y;
+  disco = 178.585;
+  x = 0.0123;
+  y = -0.00987;
+
+  palPcd ( disco, &x, &y );
+  vvd ( x, 0.01284630845735895, 1e-14, "palPcd", "x", status );
+  vvd ( y, -0.01030837922553926, 1e-14, "palPcd", "y", status );
+
+  palUnpcd ( disco, &x, &y );
+  vvd ( x, 0.0123, 1e-14, "palUnpcd", "x", status );
+  vvd ( y, -0.00987, 1e-14, "palUnpcd", "y,", status );
+
+  /* Now negative disco round trip */
+  disco = -0.3333333;
+  x = 0.0123;
+  y = -0.00987;
+  palPcd ( disco, &x, &y );
+  palUnpcd ( disco, &x, &y );
+  vvd ( x, 0.0123, 1e-14, "palUnpcd", "x", status );
+  vvd ( y, -0.00987, 1e-14, "palUnpcd", "y,", status );
+}
+
 static void t_planet( int * status ) {
   int j;
   double pv[6];
@@ -1432,7 +1557,8 @@ static void t_planet( int * status ) {
     0.2217782439275216936,
     2.852118859689216658,
     0.01452010174371893229,
-    50100.
+    50100.,
+    0.
   };
   double expectedpv[6] = {
     0.07944764084631667011, -0.04118141077419014775,
@@ -1720,7 +1846,7 @@ static void t_pm( int * status ) {
   vvd ( dec2, -0.8696617307805072, 1e-10, "palPm",
         "D", status );
 
-  /* SOFA test */
+  /* SOFA/ERFA test */
   ra1 =   0.01686756;
   dec1 = -1.093989828;
   pmr1 = -1.78323516e-5;
@@ -1739,6 +1865,15 @@ static void t_pm( int * status ) {
 
 }
 
+static void t_polmo( int *status ) {
+  double elong, phi, daz;
+
+  palPolmo( 0.7, -0.5, 1.0e-6, -2.0e-6, &elong, &phi, &daz );
+  vvd(elong, 0.7000004837322044, 1.0e-12, "palPolmo", "elong", status );
+  vvd(phi, -0.4999979467222241, 1.0e-12, "palPolmo", "phi", status );
+  vvd(daz, 1.008982781275728e-6, 1.0e-12, "palPolmo", "daz", status );
+}
+
 static void t_pvobs( int *status ) {
    double pv[6];
    double expected[6] = { -4.7683600138836167813e-06,
@@ -1878,15 +2013,26 @@ static void t_ref( int *status ) {
 
 }
 
+static void t_vers( int *status ) {
+  char verstring[32];
+
+  int ver = palVers( verstring, sizeof(verstring));
+  printf("PAL Version %s (%d)\n", verstring, ver);
+  if ( ver < 6000 ) {
+    *status = 1; /* palVers introduced at v0.6.0 */
+  }
+}
+
 /**********************************************************************/
 
 int main (void) {
 
-  /* Use the SLA and SOFA conventions */
+  /* Use the SLA and SOFA/ERFA conventions */
   int status = 0; /* Unix and SAE convention */
 
   t_addet(&status);
   t_afin(&status);
+  t_altaz(&status);
   t_ampqk(&status);
   t_aop(&status);
   t_airmas(&status);
@@ -1916,6 +2062,7 @@ int main (void) {
   t_evp(&status);
   t_fk45z(&status);
   t_fk54z(&status);
+  t_fk524(&status);
   t_flotin(&status);
   t_galeq(&status);
   t_galsup(&status);
@@ -1926,17 +2073,21 @@ int main (void) {
   t_intin(&status);
   t_prec(&status);
   t_preces(&status);
+  t_ecleq(&status);
   t_ecmat(&status);
   t_e2h(&status);
   t_map(&status);
   t_mappa(&status);
+  t_mapqk(&status);
   t_mapqkz(&status);
   t_moon(&status);
   t_nut(&status);
   t_obs(&status);
   t_pa(&status);
+  t_pcd(&status);
   t_planet(&status);
   t_pm(&status);
+  t_polmo(&status);
   t_prebn(&status);
   t_pvobs(&status);
   t_range(&status);
@@ -1952,6 +2103,6 @@ int main (void) {
   t_supgal(&status);
   t_tp(&status);
   t_vecmat(&status);
+  t_vers(&status);
   return status;
 }
-
